#include "statement_dismantlers.h"
#include <suifkernel/command_line_parsing.h>

/** Generic pass that invokes a walker class.
    Taken from basepases/transforms/transforms.cpp */

template <class walker> class WalkerModule : public PipelinablePass {
  String _description;
public:
  WalkerModule(SuifEnv *env, const LString &name,
		      const String &description ) 
    : PipelinablePass(env, name), _description(description) {}

  void initialize() {
    PipelinablePass::initialize();
    _command_line->set_description(_description);
  }
    
  virtual ~WalkerModule(void)  {}

  virtual void do_procedure_definition(ProcedureDefinition *proc_def) {
    if ( !proc_def ) return;
    walker walk(get_suif_env(), proc_def);
    proc_def->walk(walk);
  }

  Module *clone() const { return(Module*)this; }
};

extern "C" void init_transformstrl(SuifEnv *suif_env)
{
  suif_env->require_module("suifnodes");
  suif_env->require_module("esuifnodes");

  ModuleSubSystem *ms = suif_env->get_module_subsystem();

  ms->register_module(new WalkerModule<PresentStatementWalker>(
     suif_env, "dismantle_present_statements",
     "Dismantle PresentStatements to IfStatements"));
		     
}
