#include "statement_dismantlers.h"
#include <basicnodes/basic_factory.h>
#include <suifnodes/suif_factory.h>
#include <esuifnodes/esuif_factory.h>
#include <utils/expression_utils.h>

Walker::ApplyStatus PresentStatementWalker::operator () (SuifObject *o) {
  SuifObject *result = dismantle_present_statement(to<PresentStatement>(o));
  set_address(result);
  return Walker::Replaced;
}

/**
  Dismantle a present statement into a cascade of IfStatements
*/
Statement *
PresentStatementWalker::dismantle_present_statement(PresentStatement *ps) {
  Statement *result = 0;

  IfStatement *is = 0;

  suif_assert(ps->get_case_count() > 0); // Need at least one case

  for ( Iter<PredicatedStatement*> iter = ps->get_case_iterator() ;
	iter.is_valid() ; iter.next() ) {
    PredicatedStatement *prs = iter.current();
    Expression *e = prs->get_predicate();
    suif_assert(e);
    remove_suif_object(e);
    Statement *s = prs->get_body();
    if (s) remove_suif_object(s);
    IfStatement *is1 = create_if_statement( ps->get_suif_env(), e, s );
    if (is) {
      is->set_else_part(is1); // This is a later case: make it an "else" branch
    } else {
      result = is1; // This is the first case: make it the result
    }
    is = is1;
  }

  suif_assert(is); // Should have created at least one

  // Put the default case, if any, into the "else" part of the last IfStatement

  Statement *s = ps->get_default_case();
  if (s) {
    remove_suif_object(s);   
    is->set_else_part(s);
  }

  ps->get_parent()->replace(ps, result);
  return result;
}
